/*
 *
 * Copyright (c) 2004-2005 Guido Falsi <mad@madpilot.net>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: Common.c,v 1.16 2005/03/03 22:17:27 mad Exp $
 *
 */

#include <PalmOS.h>

#include "miscdefs.h"
#include "DoubleStrDouble.h"

void * GetObjectPtr(UInt16 objectID)
{
	return(FrmGetObjectPtr(FrmGetActiveForm(), FrmGetObjectIndex(FrmGetActiveForm(), objectID)));
}

FieldPtr NumFldSetInt(UInt16 field, Int32 value)
{
	FieldPtr fld;
	MemHandle handle;
	Char *p;

	fld = GetObjectPtr(field);
	handle = FldGetTextHandle(fld);

	if(!handle)
	{
		handle = MemHandleNew(maxStrIToALen * sizeof(Char));
		if(!handle) return NULL;
	} else {
		FldSetTextHandle(fld, NULL);
	}

	p = MemHandleLock(handle);
	if(!p) return NULL;
	StrIToA(p, value);
	MemHandleUnlock(handle);
	FldSetTextHandle(fld, handle);

	return fld;
}

Int32 NumFldGetInt(UInt16 field)
{
	FieldPtr fld;
	MemHandle handle;
	Char *p;
	Int32 ret;

	fld = GetObjectPtr(field);
	handle = FldGetTextHandle(fld);
	if(!handle) return 0;
	p = MemHandleLock(handle);
	if(!p) return 0;
	ret = StrAToI(p);
	MemHandleUnlock(handle);

	return ret;
}

void FldDestroy(UInt16 field)
{
	FieldPtr fld;
	MemHandle handle;

	fld = GetObjectPtr(field);
	handle = FldGetTextHandle(fld);
	FldSetTextHandle(fld, NULL);
	if(handle) MemHandleFree(handle);
}

FieldPtr NEFldNumSetInt(UInt16 field, Int32 value)
{
	FieldPtr fld;
	Char *p;

	fld = GetObjectPtr(field);
	p = FldGetTextPtr(fld);

	if(!p)
	{
		p = MemPtrNew(maxStrIToALen * sizeof(Char));
		if(!p) return NULL;
	} else {
		FldSetTextPtr(fld, "");
	}

	StrIToA(p, value);
	FldSetTextPtr(fld, p);

	return fld;
}

FieldPtr NEFldNumSetDouble(UInt16 field, double value)
{
	FieldPtr fld;
	Char *p;

	fld = GetObjectPtr(field);
	p = FldGetTextPtr(fld);

	if(!p)
	{
		p = MemPtrNew(BUFSIZE * sizeof(Char));
		if(!p) return NULL;
	} else {
		FldSetTextPtr(fld, "");
	}

	printDouble(value, p);
	FldSetTextPtr(fld, p);

	return fld;
}

void NEFldDestroy(UInt16 field)
{
	FieldPtr fld;
	Char *p;

	fld = GetObjectPtr(field);
	p = FldGetTextPtr(fld);
	FldSetTextPtr(fld, "");
	if(p) MemPtrFree(p);
}

double NumFldGetDouble(UInt16 field)
{
	FieldPtr fld;
	MemHandle tmp;
	Char *tmptxt;
	double ret;

	fld = GetObjectPtr(field);
	if(!fld) return 0;
	tmp = FldGetTextHandle(fld);
	if(!tmp) return 0;
	tmptxt = MemHandleLock(tmp);
	if(!tmptxt) return 0;
	ret = my_atof(tmptxt);
	MemHandleUnlock(tmp);

	return ret;
}

FieldPtr NumFldSetDouble(UInt16 field, double value)
{
	FieldPtr fld;
	MemHandle handle;
	Char *p;

	fld = GetObjectPtr(field);
	handle = FldGetTextHandle(fld);

	if(!handle)
	{
		handle = MemHandleNew(BUFSIZE * sizeof(Char));
		if(!handle) return NULL;
	} else {
		FldSetTextHandle(fld, NULL);
	}

	p = MemHandleLock(handle);
	if(!p) return NULL;
	printDouble(value, p);
	MemHandleUnlock(handle);
	FldSetTextHandle(fld, handle);

	return fld;
}

void FldAddPoints(UInt16 field)
{
	MemHandle tmp;
	Char *tmptxt;
	FieldPtr fld;
	Int32 counter;

 	fld = GetObjectPtr(field);
	if(!fld) return;
	tmp = FldGetTextHandle(fld);
	if(!tmp) return;
	FldSetTextHandle(fld, NULL);

	tmptxt = MemHandleLock(tmp);
	if(!tmptxt) return;
	counter = StrAToI(tmptxt);
	counter++;
	StrIToA(tmptxt, counter);
	MemHandleUnlock(tmp);

	FldSetTextHandle(fld, tmp);
	FldDrawField(fld);
	SndPlaySystemSound(sndClick);
}

/*
 * The following two functions and all of the table scrolling and managing
 * code are inspired and mostly copied from the very handy tutorial
 * at http://www.mixdown.ca/~andrew/palm.php from Akohl Smith. Many
 * thanks to him.
 */

Boolean TblScrollTable(TablePtr table, Int16 delta, Int16 *selItem, Int16 *selRow, Boolean *selIsOffScreen)
{
	RectangleType r, rs, rc;
	UInt16 numRows, i, j;

	numRows = TblGetNumberOfRows(table);

	i = (delta < 0) ? -delta : delta;
	j = (i > numRows ) ? numRows : i;

	numRows--;

	TblGetBounds(table, &r);
	WinScrollRectangle(&r, (delta < 0) ? winDown : winUp, TblGetRowHeight(table, 0) * i, &r);

	while(j--)
		TblMarkRowInvalid(table, (delta < 0) ? j : numRows - j);

	TblRedrawTable(table);

	if(*selItem != -1 && !*selIsOffScreen)
	{
		TblGetItemBounds(table, *selRow, 0, &rs);
		TblGetItemBounds(table, *selRow, 1, &rc);
		rs.extent.x += rc.extent.x; /* XXX */

		WinGetClip(&rc);
		WinSetClip(&r);
		WinClipRectangle(&rs);
		WinSetClip(&rc);

		return(rs.extent.x && rs.extent.y);
	} else {
		return false;
	}
}

void ListFormScrollTable(TablePtr tbl, ScrollBarPtr scl, Int16 *tItem, Int16 *selItem, Int16 *selRow, Boolean *selIsOffScreen, EventPtr e, UInt8 how)
{
	RectangleType rect, rict;
	Int16 delta, value, newValue = 0, min, max, psize;

	if(how == Scroller)
	{
		delta = (e->data.sclRepeat.newValue - e->data.sclRepeat.value);
	} else {
		SclGetScrollBar(scl, &value, &min, &max, &psize);

		if(how == BtnDn)
			newValue = (value + psize) > max ? max : value + psize;
		else if(how == BtnUp)
			newValue = (value - psize) < 0 ? 0 : value - psize;

		SclSetScrollBar(scl, newValue, min, max, psize);
		delta = newValue - value;
	}

	*tItem += delta;
	*selRow -= delta;

	if(*selItem - *tItem < 0 || *selItem - *tItem > TblGetNumberOfRows(tbl) - 1)
		*selIsOffScreen = true;
	else
		*selIsOffScreen = false;

	if(TblScrollTable(tbl, delta, selItem, selRow, selIsOffScreen))
	{
		/*
		 * this piece of code is not as generic as it should
		 * be, but some solution is possible in the future.
		 */

		TblGetItemBounds(tbl, *selRow, Place, &rect);
		TblGetItemBounds(tbl, *selRow, Date, &rict);
		rect.extent.x += rict.extent.x;
		WinInvertRectangle(&rect, 0);
	}
}

void TblSelectTableItem(TablePtr tbl, UInt16 numItems, Int16 *tItem, Int16 *selItem, Int16 *selRow, Int16 *selCol, Boolean *selIsOffScreen, EventPtr e)
{
	Int16 row, col;
	RectangleType rect, rict;

	if((e->data.tblEnter.row + *tItem) < numItems)
	{
		if(*selItem != -1 && !*selIsOffScreen)
		{
			TblGetItemBounds(tbl, *selRow, Place, &rect);
			TblGetItemBounds(tbl, *selRow, Date, &rict);
			rect.extent.x += rict.extent.x;
			WinInvertRectangle(&rect, 0);
		}

		row = e->data.tblEnter.row;
		col = e->data.tblEnter.column;

		TblGetItemBounds(tbl, row, Place, &rect);
		TblGetItemBounds(tbl, row, Date, &rict);
		rect.extent.x += rict.extent.x;
		WinInvertRectangle(&rect, 0);

		*selRow = row;
		*selCol = col;
		*selItem = *tItem + row;
		*selIsOffScreen = false;
	}
}
