/*
 *
 * Copyright (c) 2004-2005 Guido Falsi <mad@madpilot.net>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: CompList.c,v 1.16 2005/03/13 12:22:08 mad Exp $
 *
 */

#include <PalmOS.h>

#include "miscdefs.h"
#include "PalmSS.h"
#include "PSSdb.h"
#include "MatchList.h"
#include "PalmSSRsc.h"
#include "Common.h"
#include "StageList.h"

static char			*NPtr = NULLPTRERR;
static char			*NHdl = NULLHDLERR;

Int16				complistindex = noListSelection;

static MemHandle		editrecordH;
static Boolean			editisnew = false;

/* pointer for the table information array */

static CompTableDataPtr		TableArray = NULL;

/* global variables to track the table state */

static Int16 			selectedItem = NoRecord;
static Int16			topItem = 0;
static Int16			selectedRow = NoRecord;
static Int16			selectedCol = NoRecord;
static Boolean			selectedIsOffScreen = false;

static void CompTblDraw(void *table, Int16 row, Int16 col, RectanglePtr bounds)
{
	FontID font = 0; /* Make gcc quiet */
	UInt16 ArrayItem, numrecs;

	ArrayItem = row + topItem;
        numrecs = CompInMatchDBNumRecords(matchrecordindex);

	if(ArrayItem < numrecs)
	{
		if(col == Name)
		{
			if(TableArray[ArrayItem].DQ) /* make disqualified competitors bold */
				font = FntSetFont(boldFont);
			WinDrawChars(TableArray[ArrayItem].name, StrLen(TableArray[ArrayItem].name), bounds->topLeft.x, bounds->topLeft.y);
			if(TableArray[ArrayItem].DQ)
				FntSetFont(font);
		}
	}
}

static void CompListFormSetup(FormPtr frm)
{
	TablePtr tbl;
	ScrollBarPtr scroller;
	UInt16 numrecs, i, rows;

	selectedItem = NoRecord;
	topItem = 0;
	selectedRow = NoRecord;
	selectedCol = NoRecord;
	selectedIsOffScreen = false;
	TableArray = NULL;

	tbl = GetObjectPtr(CompsList);
	scroller = GetObjectPtr(CompsList_Scroll);
	rows = TblGetNumberOfRows(tbl);

	for(i = 0; i < rows; i++)
	{
		TblSetItemStyle(tbl, i, Name, customTableItem);
		TblSetItemStyle(tbl, i, CompUnused, customTableItem);

		TblSetRowUsable(tbl, i, true);
	}

	TblSetColumnUsable(tbl, Name, true);
	TblSetColumnUsable(tbl, CompUnused, true);

	TblSetColumnSpacing(tbl, Name, 1);

	TblSetCustomDrawProcedure(tbl, Name, CompTblDraw);
	TblSetCustomDrawProcedure(tbl, CompUnused,CompTblDraw);

	numrecs = CompInMatchDBNumRecords(matchrecordindex);

	CompDBTableDataQuery(matchrecordindex, &TableArray); /* XXX */

	SclSetScrollBar(scroller, topItem, 0, (numrecs < rows) ? 0 : (numrecs - rows), rows);
}

static void CompListFormNewComp()
{
	CompPtr newcomp;

	newcomp = MemPtrNew(sizeof(Comp));
	ErrFatalDisplayIf(newcomp == NULL, NPtr);

	/* Initialize the sctructure with default(mostly null) values */

	newcomp->num = 0;
	newcomp->division = Production;
	newcomp->PF = Minor;
	newcomp->DQ = false;
	newcomp->MtcRef = GetMatchUID(matchrecordindex);
	newcomp->ScrRef = NoRecord;
	newcomp->prev = CompDBGetLastUID(matchrecordindex);
	newcomp->next = NoRecord;
	StrCopy(newcomp->name, "");

	CompDBCreateComp(newcomp, matchrecordindex, &complistindex);
	MemPtrFree(newcomp);

	editisnew = true;

	FrmGotoForm(CompNewForm);
}

static void CompListFormDelComp()
{
	TablePtr tbl;
	ScrollBarPtr scroller;
	UInt16 numrecs, page = 0;

        complistindex = selectedItem;
        if(complistindex == NoRecord) return;

	CompDBTableDataFree(matchrecordindex, &TableArray, false);

	CompDBDeleteComp(matchrecordindex, complistindex);

	/* Now we should update the list and scroller. */
	tbl = GetObjectPtr(CompsList);
	scroller = GetObjectPtr(CompsList_Scroll);
	numrecs = CompInMatchDBNumRecords(matchrecordindex);
	page = TblGetNumberOfRows(tbl);

	if(numrecs > page)
		topItem = ((numrecs - topItem - 1) < page) ? (numrecs - page) :
topItem;

	selectedItem = NoRecord;
	selectedRow = NoRecord;
	selectedCol = NoRecord;
	selectedIsOffScreen = false;

	SclSetScrollBar(scroller, topItem, 0, (numrecs < page) ? 0 : (numrecs - page), page);
	CompDBTableDataQuery(matchrecordindex, &TableArray);
	TblMarkTableInvalid(tbl);
        TblRedrawTable(tbl);
}

Boolean CompListFormHandleEvent(EventPtr e)
{
	Boolean handled = false;
	FormPtr frm;

	switch (e->eType) {

	case frmOpenEvent:
		frm = FrmGetActiveForm();
		CompListFormSetup(frm);
		FrmDrawForm(frm);
		handled = true;
		break;

	case frmCloseEvent:
		CompDBTableDataFree(matchrecordindex, &TableArray, editisnew);
		break;

	case menuEvent:
		MenuEraseStatus(NULL);

		switch(e->data.menu.itemID) {

		case About_Menu:
			FrmCustomAlert(About_Alert, NULL, NULL, NULL);
			handled = true;
			break;

		default:
			handled = false;
			break;
		}

		break;

	case ctlSelectEvent:
		switch(e->data.ctlSelect.controlID) {
		case CLF_Prev:
			complistindex = NoRecord;
			FrmGotoForm(StageListForm);
			handled = true;
			break;

		case CLF_New:
			CompListFormNewComp();
			handled = true;
			break;

		case CLF_Edit:
			complistindex = selectedItem;
			if(complistindex == NoRecord) break;
			editisnew = false;
			FrmGotoForm(CompNewForm);
			handled = true;
			break;

		case CLF_Del:
			CompListFormDelComp();
			handled = true;
			break;

		case CLF_Next:
			complistindex = selectedItem;
			if(complistindex == NoRecord) break;
			FrmGotoForm(ScoreInputForm);
			handled = true;
			break;

		case CLF_Results:
			FrmGotoForm(ResultsForm);
			handled = true;
			break;
		}
		break;

	case sclRepeatEvent:
		ListFormScrollTable(GetObjectPtr(CompsList), GetObjectPtr(CompsList_Scroll), &topItem, &selectedItem, &selectedRow, &selectedIsOffScreen, e, Scroller);
		break;

	case tblEnterEvent:
		TblSelectTableItem(GetObjectPtr(CompsList), CompInMatchDBNumRecords(matchrecordindex), &topItem, &selectedItem, &selectedRow, &selectedCol, &selectedIsOffScreen, e);
		handled=true;
		break;

	case keyDownEvent:
		switch(e->data.keyDown.chr) {
		case pageDownChr:
			ListFormScrollTable(GetObjectPtr(CompsList), GetObjectPtr(CompsList_Scroll), &topItem, &selectedItem, &selectedRow, &selectedIsOffScreen, e, BtnDn);
			handled = true;
			break;

		case pageUpChr:
			ListFormScrollTable(GetObjectPtr(CompsList), GetObjectPtr(CompsList_Scroll), &topItem, &selectedItem, &selectedRow, &selectedIsOffScreen, e, BtnUp);
			handled = true;
			break;
		}
		break;

	default:
		break;

	}

	return handled;
}

static void CompNewFormSetup(FormPtr frm)
{
	CompPtr record;
	FieldPtr fld;
	UInt32 offset;

	editrecordH = MemHandleNew(sizeof(Comp));
	ErrFatalDisplayIf(editrecordH == NULL, NHdl);
	record = MemHandleLock(editrecordH);
	ErrFatalDisplayIf(record == NULL, NPtr);

	CompDBGetComp(matchrecordindex, complistindex, record);

	fld = GetObjectPtr(CNF_Name);
	offset = sizeof(UInt16) * 2 + sizeof(UInt32) * 4;
	FldSetText(fld, editrecordH, offset, BUFSIZE);

	switch(record->division)
	{
	case Production:
		FrmSetControlGroupSelection(frm, Div_Group, CNF_D_Prod);
		break;

	case Standard:
		FrmSetControlGroupSelection(frm, Div_Group, CNF_D_Stand);
		break;

	case Modified:
		FrmSetControlGroupSelection(frm, Div_Group, CNF_D_Mod);
		break;

	case Open:
		FrmSetControlGroupSelection(frm, Div_Group, CNF_D_Open);
		break;

	}

	switch(record->PF)
	{
	case Minor:
		FrmSetControlGroupSelection(frm, PF_Group, CNF_PF_Minor);
		break;

	case Major:
		FrmSetControlGroupSelection(frm, PF_Group, CNF_PF_Major);
		break;

	}

	FrmSetControlValue(frm, FrmGetObjectIndex(frm, CNF_DQ_CBox), record->DQ);

	MemHandleUnlock(editrecordH);

	FrmSetFocus(frm, FrmGetObjectIndex(frm, CNF_Name));
}

static void CompNewFormSaveRecord()
{
	CompPtr record;
	FormPtr frm;
	UInt16 sel;

	record = MemHandleLock(editrecordH);
	ErrFatalDisplayIf(record == NULL, NPtr);

	/*
	 * Get the values back to the unpacked structures and pass
	 * to the specialized function to pack in the datagbase.
	 */

	frm = FrmGetActiveForm();
	sel = FrmGetObjectId(frm, FrmGetControlGroupSelection(frm, Div_Group));

	switch(sel)
	{
	case CNF_D_Prod:
		record->division = Production;
		break;

	case CNF_D_Stand:
		record->division = Standard;
		break;

	case CNF_D_Mod:
		record->division = Modified;
		break;

	case CNF_D_Open:
		record->division = Open;
		break;

	}

	sel = FrmGetObjectId(frm, FrmGetControlGroupSelection(frm, PF_Group));

	switch(sel)
	{
	case CNF_PF_Minor:
		record->PF = Minor;
		break;

	case CNF_PF_Major:
		record->PF = Major;
		break;
	}

	if(FrmGetControlValue(frm, FrmGetObjectIndex(frm, CNF_DQ_CBox)))
		record->DQ = true;
	else
		record->DQ = false;

	CompDBUpdateComp(record, matchrecordindex, complistindex);
	MemHandleUnlock(editrecordH);
}

Boolean CompNewFormHandleEvent(EventPtr e)
{
	Boolean handled = false;
	FormPtr frm;

	switch (e->eType) {

	case frmOpenEvent:
		frm = FrmGetActiveForm();
		CompNewFormSetup(frm);
		FrmDrawForm(frm);
		handled = true;
		break;

	case frmCloseEvent:
		if(editisnew) CompDBDeleteComp(matchrecordindex, complistindex);
		FldDestroy(CNF_Name);
		editisnew = false;
		break;

	case menuEvent:
		MenuEraseStatus(NULL);

		switch(e->data.menu.itemID) {

		case About_Menu:
			FrmCustomAlert(About_Alert, NULL, NULL, NULL);
			handled = true;
			break;

		default:
			handled = false;
			break;
		}

		break;

	case ctlSelectEvent:
		switch(e->data.ctlSelect.controlID) {

		case CNF_Cancel:
			FrmGotoForm(CompListForm);
			handled = true;
			break;

		case CNF_Ok:
			CompNewFormSaveRecord();
			editisnew = false;
			FrmGotoForm(CompListForm);
			handled = true;
			break;

		}
		break;

	default:
		break;

	}

	return handled;
}
