/*
 *
 * Copyright (c) 2004-2005 Guido Falsi <mad@madpilot.net>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: MatchList.c,v 1.26 2005/03/13 12:22:08 mad Exp $
 *
 */

#include <PalmOS.h>

#include "miscdefs.h"
#include "PalmSS.h"
#include "PSSdb.h"
#include "PalmSSRsc.h"
#include "Common.h"

/* PopUpList selection options */
#define Today			0
#define	Tomorrow		1
#define Choose			2

static char			*NPtr = NULLPTRERR;
static char			*NHdl = NULLHDLERR;

UInt16				matchrecordindex = NoRecord;

static MemHandle		editrecordH;
static Boolean			editisnew = false;

/* pointer for the table information array */

static MatchTableDataPtr	TableArray = NULL;

/* global variables to track the table state */

static Int16			selectedItem = NoRecord;
static Int16			topItem = 0;
static Int16			selectedRow = NoRecord;
static Int16			selectedCol = NoRecord;
static Boolean			selectedIsOffScreen = false;

static void MatchTblDraw(void *table, Int16 row, Int16 col, RectanglePtr bounds)
{
	Char datestr[dateStringLength];
	UInt16 len, width, ArrayItem, numrecs;
	Coord x;

	ArrayItem = row + topItem;
	numrecs = MatchDBNumRecords();
	
	if(ArrayItem < numrecs)
	{
		if(col == Place)
		{
			WinDrawChars(TableArray[ArrayItem].place, StrLen(TableArray[ArrayItem].place), bounds->topLeft.x, bounds->topLeft.y);
		} else if(col == Date) {
			DateToAscii(TableArray[ArrayItem].date->month, TableArray[ArrayItem].date->day, TableArray[ArrayItem].date->year + 1904, ShortDateFormat, datestr);
			len = StrLen(datestr);
			width = FntCharsWidth(datestr, len);
			x = bounds->topLeft.x + bounds->extent.x - width - 2;
			WinDrawChars(datestr, len, x, bounds->topLeft.y);
		}
	}
}

static void MatchListFormSetup(FormPtr frm)
{
	TablePtr tbl;
	ScrollBarPtr scroller;
	UInt16 numrecs, i, rows;

	selectedItem = NoRecord;
	topItem = 0;
	selectedRow = NoRecord;
	selectedCol = NoRecord;
	selectedIsOffScreen = false;
	TableArray = NULL;

	tbl = GetObjectPtr(MatchesList);
	scroller = GetObjectPtr(MatchesList_Scroll);
	rows = TblGetNumberOfRows(tbl);

	for(i = 0; i < rows; i++)
	{
		TblSetItemStyle(tbl, i, Place, customTableItem);
		TblSetItemStyle(tbl, i, Date, customTableItem);

		TblSetRowUsable(tbl, i, true);
	}

	TblSetColumnUsable(tbl, Place, true);
	TblSetColumnUsable(tbl, Date, true);

	TblSetColumnSpacing(tbl, Place, 2);

	TblSetCustomDrawProcedure(tbl, Place, MatchTblDraw);
	TblSetCustomDrawProcedure(tbl, Date, MatchTblDraw);

	ResetCounters();
	numrecs = MatchDBNumRecords();

	MatchDBTableDataQuery(&TableArray); /* XXX */

	SclSetScrollBar(scroller, topItem, 0, (numrecs < rows) ? 0 : (numrecs - rows), rows);
}

static void MatchListFormNewMatch()
{
	MatchPtr newmatch;

	newmatch = MemPtrNew(sizeof(Match));
	ErrFatalDisplayIf(newmatch == NULL, NPtr);

	/* Initialize the sctructure with default(mostly null) values */

	DateSecondsToDate(TimGetSeconds(), &(newmatch->date));
	newmatch->StgRef = NoRecord;
	newmatch->CptRef = NoRecord;
	newmatch->StgCnt = 0;
	newmatch->CptCnt = 0;
	StrCopy(newmatch->place, "");
	StrCopy(newmatch->name, "");

	MatchDBCreateMatch(newmatch, &matchrecordindex);
	MemPtrFree(newmatch);

	editisnew = true;

	FrmGotoForm(MatchNewForm);
}

static void MatchListFormDelMatch()
{
	TablePtr tbl;
	ScrollBarPtr scroller;
	UInt16 numrecs, page = 0;

	matchrecordindex = selectedItem;
	if(matchrecordindex == NoRecord) return;

	MatchDBTableDataFree(&TableArray, false);

	MatchDBDeleteMatch(matchrecordindex);

	/* Now we should update the list and scroller. */
	tbl = GetObjectPtr(MatchesList);
	scroller = GetObjectPtr(MatchesList_Scroll);
	numrecs = MatchDBNumRecords();

	page = TblGetNumberOfRows(tbl);
	if(numrecs > page)
		topItem = ((numrecs - topItem - 1) < page) ? (numrecs - page) : topItem;
	selectedItem = NoRecord;
	selectedRow = NoRecord;
	selectedCol = NoRecord;
	selectedIsOffScreen = false;
	SclSetScrollBar(scroller, topItem, 0, (numrecs < page) ? 0 : (numrecs - page), page);
	MatchDBTableDataQuery(&TableArray);
	TblMarkTableInvalid(tbl);
	TblRedrawTable(tbl);
}

Boolean MatchListFormHandleEvent(EventPtr e)
{
	Boolean handled = false;
	FormPtr frm;

	switch (e->eType) {

	case frmOpenEvent:
		frm = FrmGetActiveForm();
		MatchListFormSetup(frm);
		FrmDrawForm(frm);
		handled = true;
		break;

	case frmCloseEvent:
		MatchDBTableDataFree(&TableArray, editisnew);
		break;

	case menuEvent:
		MenuEraseStatus(NULL);

		switch(e->data.menu.itemID) {

		case About_Menu:
			FrmCustomAlert(About_Alert, NULL, NULL, NULL);
			handled = true;
			break;

		default:
			handled = false;
			break;
		}

		break;

	case ctlSelectEvent:
		switch(e->data.ctlSelect.controlID) {
		case MLF_New:
			MatchListFormNewMatch();
			handled = true;
			break;

		case MLF_Edit:
			matchrecordindex = selectedItem;
			if(matchrecordindex == NoRecord) break;
			editisnew = false;
			FrmGotoForm(MatchNewForm);
			handled = true;
			break;

		case MLF_Del:
			MatchListFormDelMatch();
			handled = true;
			break;

		case MLF_Next:
			matchrecordindex = selectedItem;
			if(matchrecordindex == NoRecord) break;
			FrmGotoForm(StageListForm);
			handled = true;
			break;

		case MLF_Results:
			matchrecordindex = selectedItem;
			if(matchrecordindex == NoRecord) break;
			FrmGotoForm(ResultsForm);
			handled = true;
			break;
		}
		break;

	case sclRepeatEvent:
		ListFormScrollTable(GetObjectPtr(MatchesList), GetObjectPtr(MatchesList_Scroll), &topItem, &selectedItem, &selectedRow, &selectedIsOffScreen, e, Scroller);
		break;

	case tblEnterEvent:
		TblSelectTableItem(GetObjectPtr(MatchesList), MatchDBNumRecords(), &topItem, &selectedItem, &selectedRow, &selectedCol, &selectedIsOffScreen, e);
		handled=true;
		break;

	case keyDownEvent:
		switch(e->data.keyDown.chr) {
		case pageDownChr:
			ListFormScrollTable(GetObjectPtr(MatchesList), GetObjectPtr(MatchesList_Scroll), &topItem, &selectedItem, &selectedRow, &selectedIsOffScreen, e, BtnDn);
			handled = true;
			break;

		case pageUpChr:
			ListFormScrollTable(GetObjectPtr(MatchesList), GetObjectPtr(MatchesList_Scroll), &topItem, &selectedItem, &selectedRow, &selectedIsOffScreen, e, BtnUp);
			handled = true;
			break;
		}
		break;

	default:
		break;

	}

	return handled;
}

static void MatchNewFormSetup(FormPtr frm)
{
	MatchPtr record;
	FieldPtr fld;
	ControlPtr ctl;
	Char *datestr, *nameP;
	UInt32 offset;
	MemHandle nameH;

	editrecordH = MemHandleNew(sizeof(Match));
	ErrFatalDisplayIf(editrecordH == NULL, NHdl);
	record = (MatchPtr)MemHandleLock(editrecordH);
	ErrFatalDisplayIf(record == NULL, NPtr);

	MatchDBGetMatch(matchrecordindex, record);

	ctl = GetObjectPtr(MNF_Date_Trigger);
	datestr = (Char *)CtlGetLabel(ctl);
	DateToAscii(record->date.month, record->date.day, record->date.year +1904, ShortDateFormat, datestr);
	CtlSetLabel(ctl, datestr);

	fld = GetObjectPtr(MNF_Place);
	offset = sizeof(DateType) + sizeof(UInt32) * 2 + sizeof(UInt16) * 2;
	FldSetText(fld, editrecordH, offset, BUFSIZE);

	/*
	 * Field code seems to have big problems with multiple
	 * FldSetText called pointing to the same handle with different
	 * offsets(which is also obvious, but considering why such
	 * a call was created looks a little ridicolous...) So I'll
	 * have to waste memory to copy the name string to a new
	 * handle and then take it back when saving the form.
	 */

	nameH = MemHandleNew(BUFSIZE * sizeof(Char));
	ErrFatalDisplayIf(nameH == NULL, NHdl);
	nameP = MemHandleLock(nameH);
	ErrFatalDisplayIf(nameP == NULL, NPtr);
	StrCopy(nameP, record->name);
	MemHandleUnlock(nameH);
	fld = GetObjectPtr(MNF_Name);
	FldSetTextHandle(fld, nameH);

	NEFldNumSetInt(MNF_Stages, record->StgCnt);

	NEFldNumSetInt(MNF_Comps, record->CptCnt);

	MemHandleUnlock(editrecordH);

	/* Last important thing. Give focus to one of the Fields */
	FrmSetFocus(frm, FrmGetObjectIndex(frm, MNF_Place));
}

static void MatchNewFormDateCh(Int16 selection)
{
	ControlPtr ctl;
	UInt16 newmonth, newday, newyear;
	Char *datestr;
	MatchPtr match;

	ctl = GetObjectPtr(MNF_Date_Trigger);
	match = (MatchPtr)MemHandleLock(editrecordH);
	ErrFatalDisplayIf(match == NULL, NPtr);

	switch (selection) {

	case Today:
		/* get today's date, save in editing chunk */
		DateSecondsToDate(TimGetSeconds(), &match->date);
		/* and write it in the control on screen */
		datestr = (Char *)CtlGetLabel(ctl);
		DateToAscii(match->date.month, match->date.day, match->date.year + 1904, ShortDateFormat, datestr);
		CtlSetLabel(ctl, datestr);
		break;

	case Tomorrow:
		/* add 24 hours to today's time (86400 seconds in a day) */
		DateSecondsToDate(TimGetSeconds() + 86400, &match->date);
		datestr = (Char *)CtlGetLabel(ctl);
		DateToAscii(match->date.month, match->date.day, match->date.year + 1904, ShortDateFormat, datestr);
		CtlSetLabel(ctl, datestr);
		break;

	case Choose:
		newmonth = match->date.month;
		newday = match->date.day;
		newyear = match->date.year + 1904;
		if(SelectDay(selectDayByDay, &newmonth, &newday, &newyear, "Match Date"))
		{
			match->date.month = newmonth;
			match->date.day = newday;
			match->date.year = newyear - 1904;
			datestr = (Char *)CtlGetLabel(ctl);
			DateToAscii(match->date.month, match->date.day, match->date.year + 1904, ShortDateFormat, datestr);
			CtlSetLabel(ctl, datestr);
		}
		break;
	}

	MemHandleUnlock(editrecordH);
}

static void MatchNewFormSaveMatch()
{
	MatchPtr record;
	MemHandle nameH;
	Char *nameP;
	FieldPtr fld;

	record = MemHandleLock(editrecordH);
	ErrFatalDisplayIf(record == NULL, NPtr);

	/*
	 * First of all we need to retrive the name string we had
	 * to allocate a separate handle for in MatchNewFormSetup().
	 */

	fld = GetObjectPtr(MNF_Name);
	nameH = FldGetTextHandle(fld);
	nameP = MemHandleLock(nameH);
	ErrFatalDisplayIf(nameP == NULL, NPtr);
	StrCopy(record->name, nameP);
	MemHandleUnlock(nameH);

	/* take the editing record pointer and store it in DB */
	MatchDBUpdateMatch(record, &matchrecordindex);
	MemHandleUnlock(editrecordH);
}

Boolean MatchNewFormHandleEvent(EventPtr e)
{
	Boolean handled = false;
	FormPtr frm;

	switch (e->eType) {

	case frmOpenEvent:
		frm = FrmGetActiveForm();
		MatchNewFormSetup(frm);
		FrmDrawForm(frm);
		handled = true;
		break;

	case frmCloseEvent:
		if(editisnew) MatchDBDeleteMatch(matchrecordindex);

		NEFldDestroy(MNF_Stages);
		NEFldDestroy(MNF_Comps);
		FldDestroy(MNF_Name);
		FldDestroy(MNF_Place);

		editisnew = false;
		break;

	case menuEvent:
		MenuEraseStatus(NULL);

		switch(e->data.menu.itemID) {

		case About_Menu:
			FrmCustomAlert(About_Alert, NULL, NULL, NULL);
			handled = true;
			break;

		default:
			handled = false;
			break;
		}

		break;

	case popSelectEvent:
		if(e->data.popSelect.listID == MNF_Date_List)
		{
			MatchNewFormDateCh(e->data.popSelect.selection);
			handled = true;
		}
		break;

	case ctlSelectEvent:
		switch(e->data.ctlSelect.controlID) {

		case MNF_Cancel:
			FrmGotoForm(MatchListForm);
			handled = true;
			break;

		case MNF_Ok:
			/* save the record in the DB and exit */
			MatchNewFormSaveMatch();
			editisnew = false;
			FrmGotoForm(MatchListForm);
			handled=true;
			break;
		}
		break;

	default:
		break;

	}

	return handled;
}
