/*
 *
 * Copyright (c) 2004-2005 Guido Falsi <mad@madpilot.net>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: PalmSS.c,v 1.15 2005/03/12 05:02:42 mad Exp $
 *
 */

#include <PalmOS.h>

#include "miscdefs.h"
#include "PalmSSRsc.h"
#include "ScoreInputForm.h"
#include "MatchList.h"
#include "StageList.h"
#include "CompList.h"
#include "Results.h"
#include "Common.h"
#include "PSSdb.h"

/* Global variables */

NumberFormatType numberFormat;
DateFormatType LongDateFormat;
DateFormatType ShortDateFormat;

/*
 * Should add some code to check ROM version. 2.0 is the minimum required
 */

static Boolean ApplicationHandleEvent(EventPtr e)
{
	FormPtr frm;
	UInt16 formId;
	Boolean handled = false;

	if (e->eType == frmLoadEvent) {
		formId = e->data.frmLoad.formID;
		frm = FrmInitForm(formId);
		FrmSetActiveForm(frm);

		switch(formId) {
			case ScoreInputForm:
				FrmSetEventHandler(frm, ScoreInputFormHandleEvent);
				break;

			case MatchListForm:
				FrmSetEventHandler(frm, MatchListFormHandleEvent);
				break;

			case MatchNewForm:
				FrmSetEventHandler(frm, MatchNewFormHandleEvent);
				break;

			case StageListForm:
				FrmSetEventHandler(frm, StageListFormHandleEvent);
				break;

			case StageNewForm:
				FrmSetEventHandler(frm, StageNewFormHandleEvent);
				break;

			case CompListForm:
				FrmSetEventHandler(frm, CompListFormHandleEvent);
				break;

			case CompNewForm:
				FrmSetEventHandler(frm, CompNewFormHandleEvent);
				break;

			case ResultsForm:
				FrmSetEventHandler(frm, ResultsFormHandleEvent);
				break;
		}
		handled = true;
	}

	return handled;
}

/* Get preferences, open (or create) app database */
static Err StartApplication(void)
{
	Err err;

	numberFormat = (TimeFormatType)PrefGetPreference(prefNumberFormat);
	ShortDateFormat = (DateFormatType)PrefGetPreference(prefDateFormat);
	LongDateFormat = (DateFormatType)PrefGetPreference(prefLongDateFormat);

	err = OpenDBs();
	if(err) return err;

	FrmGotoForm(MatchListForm);
	return 0;
}

/* Save preferences, close forms, close app database */

static void StopApplication(void)
{
	FrmSaveAllForms();
	FrmCloseAllForms();
	CloseDBs();
}

/* The main event loop */
static void EventLoop(void)
{
	Err err;
	EventType e;

	do {
		EvtGetEvent(&e, evtWaitForever);
		if (! SysHandleEvent (&e))
			if (! MenuHandleEvent (NULL, &e, &err))
				if (! ApplicationHandleEvent (&e))
					FrmDispatchEvent (&e);
	} while (e.eType != appStopEvent);
}

/* Main entry point; it is unlikely you will need to change this except to
   handle other launch command codes */
UInt32 PilotMain(UInt16 cmd, void *cmdPBP, UInt16 launchFlags)
{
	Err err;

	switch (cmd)
	{
		case sysAppLaunchCmdNormalLaunch:
			err = StartApplication();
			if (err) return err;

			EventLoop();
			StopApplication();
			break;

		default:
			return sysErrParamErr;
			break;
	}

	return errNone;
}
