/*
 *
 * Copyright (c) 2005 Guido Falsi <mad@madpilot.net>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: Results.c,v 1.4 2005/03/13 17:28:14 mad Exp $
 *
 */

#include <PalmOS.h>

#include "miscdefs.h"
#include "PSSdb.h"
#include "MatchList.h"
#include "StageList.h"
#include "CompList.h"
#include "DoubleStrDouble.h"
#include "PalmSSRsc.h"
#include "Common.h"

static char			*NPtr = NULLPTRERR;
/* static char			*NHdl = NULLHDLERR; */

/* pointer for the table information array */

static ResultTableDataPtr	*TableArray;

/* global variables to track the table state */

static Int16			selectedItem = NoRecord;
static Int16			topItem = 0;
static Int16			selectedRow = NoRecord;
static Int16			selectedCol = NoRecord;
static Boolean			selectedIsOffScreen = false;

static void ResultsTblDraw(void *table, Int16 row, Int16 col, RectanglePtr bounds)
{
	Char str[BUFSIZE];
	UInt16 len, width, ArrayItem, numrecs;
	Coord x;

	ArrayItem = row + topItem;
	numrecs = CompInMatchDBNumRecords(matchrecordindex);

	if(ArrayItem < numrecs)
	{
		if(col == Position)
		{
			StrIToA(str, TableArray[ArrayItem]->position);
			len = StrLen(str);
			width = FntCharsWidth(str, len);
			x = bounds->topLeft.x + bounds->extent.x - width - 1;
			WinDrawChars(str, len, x, bounds->topLeft.y);
		} else if(col == RName) {
			WinDrawChars(TableArray[ArrayItem]->name, StrLen(TableArray[ArrayItem]->name), bounds->topLeft.x, bounds->topLeft.y);
		} else if(col == Percent) {
			printDouble(TableArray[ArrayItem]->percent, str);
			StrCat(str, "%");
			len = StrLen(str);
			width = FntCharsWidth(str, len);
			x = bounds->topLeft.x + bounds->extent.x - width - 4;
			WinDrawChars(str, len, x, bounds->topLeft.y);
		}
	}
}

static Int16 StageScoreCompar(void *p1, void *p2, Int32 notused)
{
	if((*(ScoreTableDataPtr *)p1)->HF > (*(ScoreTableDataPtr *)p2)->HF)
		return -1;
	else if((*(ScoreTableDataPtr *)p1)->HF < (*(ScoreTableDataPtr *)p2)->HF)
		return 1;
	else
		return 0;
}

static Int16 TableArrayCompar(void *p1, void *p2, Int32 notused)
{
	if((*(ResultTableDataPtr *)p1)->matchpoints > (*(ResultTableDataPtr *)p2)->matchpoints)
		return -1;
	else if((*(ResultTableDataPtr *)p1)->matchpoints < (*(ResultTableDataPtr *)p2)->matchpoints)
		return 1;
	else
		return 0;
}

static void ResultsTableDataCompute()
{
	UInt16 numcomps, i, j;
	Int16 numstages;
	Boolean major = false;
	Int32 points, maxpts;
	ScoreTableDataPtr *StageScoreTable;
	ScorePtr record;
	double mHF;

	StageScoreTable = NULL;
	numstages = StageInMatchDBNumRecords(matchrecordindex);
	numcomps = CompInMatchDBNumRecords(matchrecordindex);

	ResultsTableDataQuery(matchrecordindex, &TableArray);

	if(TableArray)
	{
		for(i = 0; i < numstages; i++)
		{
			ScoreTableDataQuery(matchrecordindex, i, &StageScoreTable);
			/* Compute all hit factors for the stage */
			for(j = 0; j < numcomps; j++)
			{
				major = CompDBIsCompMajor(matchrecordindex, j);

				record = MemHandleLock(StageScoreTable[j]->ScoreDBrH);
				ErrFatalDisplayIf(record == NULL, NPtr);

				points = 0;
				points += record->As * 5;
				points += record->Cs * (major ? 4 : 3);
				points += record->Ds * (major ? 2 : 1);
				points -= record->PTs * 10 + record->Misses * 10 + record->Procs * 10;
				points = points < 0 ? 0 : points;

				if(record->Time > 0.0)
					StageScoreTable[j]->HF = (double)points / record->Time;
				else
					StageScoreTable[j]->HF = 0.0;

				MemHandleUnlock(StageScoreTable[j]->ScoreDBrH);
			}

			/* sort the stage scores */
			if(numcomps > 1)
				SysQSort(StageScoreTable, numcomps, sizeof(ScoreTableDataPtr), StageScoreCompar, 0);

			maxpts = StageDBGetStageMpts(matchrecordindex, i);

			/* fill up match points based on Hit Factors */

			/* first competitor sets the standard */
		
			if(StageScoreTable[0]->HF > 0.0)
				TableArray[StageScoreTable[0]->comp]->matchpoints += (double)maxpts;
			mHF = StageScoreTable[0]->HF;

			/* others follow proportionally */

			for(j = 1; j < numcomps; j++)
				if(mHF > 0.0)
					TableArray[StageScoreTable[j]->comp]->matchpoints += (StageScoreTable[j]->HF * (double)maxpts) / mHF;

			ScoreTableDataFree(matchrecordindex, &StageScoreTable);
		}

		/*
		 * Sort the results so they're in the right order for display
		 * and easier to manage for the following stages
		 */
		if(numcomps > 1)
			SysQSort(TableArray, numcomps, sizeof(ResultTableDataPtr), TableArrayCompar, 0);

		TableArray[0]->position = 1;
		mHF = TableArray[0]->matchpoints;	/*
							 * inappropriate
							 * use of the variable.
							 * But why allocate
							 * another one if
							 * this one is not
							 * going to be used
							 * anymore?
							 */
		if(mHF > 0.0) TableArray[0]->percent = 100.0;

		/* fill up the table for pretty printing */

		for(i = 1; i < numcomps; i++)
		{
			TableArray[i]->position = i + 1;
			if(mHF > 0.0)
				TableArray[i]->percent = (TableArray[i]->matchpoints * 100.0) / mHF;
		}
	}
}

static void ResultsFormSetup(FormPtr frm)
{
	TablePtr tbl;
	ScrollBarPtr scroller;
	UInt16 numrecs, i, rows;

	selectedItem = NoRecord;
	topItem = 0;
	selectedRow = NoRecord;
	selectedCol = NoRecord;
	selectedIsOffScreen = false;
	TableArray = NULL;

	tbl = GetObjectPtr(ResultsList);
	scroller = GetObjectPtr(ResultsList_Scroll);
	rows = TblGetNumberOfRows(tbl);

	for(i = 0; i < rows; i++)
	{
		TblSetItemStyle(tbl, i, Position, customTableItem);
		TblSetItemStyle(tbl, i, RName, customTableItem);
		TblSetItemStyle(tbl, i, Percent, customTableItem);

		TblSetRowUsable(tbl, i, true);
	}

	TblSetColumnUsable(tbl, Position, true);
	TblSetColumnUsable(tbl, RName, true);
	TblSetColumnUsable(tbl, Percent, true);

	TblSetColumnSpacing(tbl, Position, 2);
	TblSetColumnSpacing(tbl, RName, 2);

	TblSetCustomDrawProcedure(tbl, Position, ResultsTblDraw);
	TblSetCustomDrawProcedure(tbl, RName, ResultsTblDraw);
	TblSetCustomDrawProcedure(tbl, Percent, ResultsTblDraw);

	numrecs = CompInMatchDBNumRecords(matchrecordindex);

	ResultsTableDataCompute();

	SclSetScrollBar(scroller, topItem, 0, (numrecs < rows) ? 0 : (numrecs - rows), rows);
}

Boolean ResultsFormHandleEvent(EventPtr e)
{
	Boolean handled = false;
	FormPtr frm;

	switch (e->eType) {

	case frmOpenEvent:
		frm = FrmGetActiveForm();
		ResultsFormSetup(frm);
		FrmDrawForm(frm);
		handled = true;
		break;

	case frmCloseEvent:
		ResultsTableDataFree(matchrecordindex, &TableArray);
		break;

	case menuEvent:
		MenuEraseStatus(NULL);

		switch(e->data.menu.itemID) {

		case About_Menu:
			FrmCustomAlert(About_Alert, NULL, NULL, NULL);
			handled = true;
			break;

		default:
			handled = false;
			break;
		}

		break;

	case ctlSelectEvent:
		switch(e->data.ctlSelect.controlID) {

		case RLF_Matches:
			FrmGotoForm(MatchListForm);
			handled = true;
			break;

		}
		break;

	case sclRepeatEvent:
		ListFormScrollTable(GetObjectPtr(ResultsList), GetObjectPtr(ResultsList_Scroll), &topItem, &selectedItem, &selectedRow, &selectedIsOffScreen, e, Scroller);
		break;

	case tblEnterEvent:
		TblSelectTableItem(GetObjectPtr(ResultsList), StageInMatchDBNumRecords(matchrecordindex), &topItem, &selectedItem, &selectedRow, &selectedCol, &selectedIsOffScreen, e);
		handled=true;
		break;

	case keyDownEvent:
		switch(e->data.keyDown.chr) {
		case pageDownChr:
			ListFormScrollTable(GetObjectPtr(ResultsList), GetObjectPtr(ResultsList_Scroll), &topItem, &selectedItem, &selectedRow, &selectedIsOffScreen, e, BtnDn);
			handled = true;
			break;

		case pageUpChr:
			ListFormScrollTable(GetObjectPtr(ResultsList), GetObjectPtr(ResultsList_Scroll), &topItem, &selectedItem, &selectedRow, &selectedIsOffScreen, e, BtnUp);
			handled = true;
			break;
		}
		break;

	default:
		break;

	}

	return handled;
}
