/*
 *
 * Copyright (c) 2004-2005 Guido Falsi <mad@madpilot.net>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: ScoreInputForm.c,v 1.15 2005/03/13 04:05:17 mad Exp $
 *
 */

#include <PalmOS.h>

#include "miscdefs.h"
#include "PalmSS.h"
#include "PSSdb.h"
#include "MatchList.h"
#include "StageList.h"
#include "CompList.h"
#include "PalmSSRsc.h"
#include "DoubleStrDouble.h"
#include "Common.h"

static char			*NPtr = NULLPTRERR;
static char			*NHdl = NULLHDLERR;

static Boolean			PFMajor = false;

static MemHandle		editrecordH;

static void ScoreInputFormRecompute()
{
	Int32 points = 0;
	double result, time;
	FieldPtr fld;

	time = NumFldGetDouble(SIF_TCount);

	points += NumFldGetInt(SIF_ACount) * 5;
	points += NumFldGetInt(SIF_CCount) * (PFMajor  ? 4 : 3);
	points += NumFldGetInt(SIF_DCount) * (PFMajor  ? 2 : 1);
	points -= NumFldGetInt(SIF_MCount) * 10 + NumFldGetInt(SIF_NCount) * 10 + NumFldGetInt(SIF_PCount) * 10;

	points = points < 0 ? 0 : points;

	fld = NEFldNumSetInt(SIF_Points, points);
	FldDrawField(fld);

	if(time > 0.0) 
		result = (double)points / time;
	else
		result = 0.0;

	fld = NEFldNumSetDouble(SIF_HFResult, result);
	FldDrawField(fld);
}

static void ScoreInputFormReset()
{
	FieldPtr fld;
	UInt16 field;

	/* XXX */
	for(field = SIF_ACount; field <= SIF_TCount; field++) {
		fld = NumFldSetInt(field, 0);
		FldDrawField(fld);
	}

	fld = NEFldNumSetInt(SIF_Points, 0);
	FldDrawField(fld);

	fld = NEFldNumSetInt(SIF_HFResult, 0);
	FldDrawField(fld);
}

static void ScoreInputFormSetup(FormPtr frm)
{
	ScorePtr record;
	CompPtr Crecord;
	Int16 width, len;
	Boolean fits;

	editrecordH = MemHandleNew(sizeof(Score));
	ErrFatalDisplayIf(editrecordH == NULL, NHdl);
	record = MemHandleLock(editrecordH);
	ErrFatalDisplayIf(record == NULL, NPtr);
	
	ScoreDBGetScore(matchrecordindex, stagelistindex, complistindex, record);

	Crecord = MemPtrNew(sizeof(Comp));
	ErrFatalDisplayIf(Crecord == NULL, NPtr);

	CompDBGetComp(matchrecordindex, complistindex, Crecord);

	PFMajor = CompDBIsCompMajor(matchrecordindex, complistindex);

	/* Set the fields to the values already scored, if any */

	NumFldSetInt(SIF_ACount, record->As);
	NumFldSetInt(SIF_CCount, record->Cs);
	NumFldSetInt(SIF_DCount, record->Ds);
	NumFldSetInt(SIF_MCount, record->Misses);
	NumFldSetInt(SIF_NCount, record->PTs);
	NumFldSetInt(SIF_PCount, record->Procs);
	NumFldSetDouble(SIF_TCount, record->Time);

	/*
	 * Give Focus to a field to avoid crash(is it a requirement
	 * of the OS or is it a "feature"?)
	 */
	FrmSetFocus(frm, FrmGetObjectIndex(frm, SIF_ACount));

	/* this needs to be here due to the following WinDrawLine() */

	FrmDrawForm(frm);
	WinDrawLine(70,102,155,102);
	WinDrawLine(70,136,155,136);
	width = 120;
	len = StrLen(Crecord->name);
	FntCharsInWidth(Crecord->name, &width, &len, &fits);
	WinDrawChars(Crecord->name, len, 40, 2);
	MemPtrFree(Crecord);
	ScoreInputFormRecompute(); /* recompute points and Hit Factor */
}

static void TargetHit(PointType tap)
{
	RectangleType area;
	Coord x;
	Coord y;

	/*
	** Check tap in the A area
	*/

	RctSetRectangle(&area, 30, 19, 7, 49);
	if(RctPtInRectangle(tap.x, tap.y, &area)) {
		FldAddPoints(SIF_ACount);
		return;
	}
	RctSetRectangle(&area, 22, 42, 23, 16);
	if(RctPtInRectangle(tap.x, tap.y, &area)) {
		FldAddPoints(SIF_ACount);
		return;
	}

	RctSetRectangle(&area, 25, 33, 5, 9);
	if(RctPtInRectangle(tap.x, tap.y, &area)) {
		FldAddPoints(SIF_ACount);
		return;
	}

	RctSetRectangle(&area, 37, 33, 5, 9);
	if(RctPtInRectangle(tap.x, tap.y, &area)) {
		FldAddPoints(SIF_ACount);
		return;
	}

	RctSetRectangle(&area, 25, 58, 5, 4);
	if(RctPtInRectangle(tap.x, tap.y, &area)) {
		FldAddPoints(SIF_ACount);
		return;
	}

	RctSetRectangle(&area, 37, 58, 5, 4);
	if(RctPtInRectangle(tap.x, tap.y, &area)) {
		FldAddPoints(SIF_ACount);
		return;
	}

	RctSetRectangle(&area, 26, 30, 4, 3);
	if(RctPtInRectangle(tap.x, tap.y, &area)) {
		FldAddPoints(SIF_ACount);
		return;
	}

	RctSetRectangle(&area, 37, 30, 4, 3);
	if(RctPtInRectangle(tap.x, tap.y, &area)) {
		FldAddPoints(SIF_ACount);
		return;
	}

	RctSetRectangle(&area, 28, 62, 2, 4);
	if(RctPtInRectangle(tap.x, tap.y, &area)) {
		FldAddPoints(SIF_ACount);
		return;
	}

	RctSetRectangle(&area, 37, 62, 2, 4);
	if(RctPtInRectangle(tap.x, tap.y, &area)) {
		FldAddPoints(SIF_ACount);
		return;
	}

	RctSetRectangle(&area, 27, 27, 3, 3);
	if(RctPtInRectangle(tap.x, tap.y, &area)) {
		FldAddPoints(SIF_ACount);
		return;
	}

	RctSetRectangle(&area, 37, 27, 3, 3);
	if(RctPtInRectangle(tap.x, tap.y, &area)) {
		FldAddPoints(SIF_ACount);
		return;
	}

	RctSetRectangle(&area, 26, 62, 2, 2);
	if(RctPtInRectangle(tap.x, tap.y, &area)) {
		FldAddPoints(SIF_ACount);
		return;
	}

	RctSetRectangle(&area, 39, 62, 2, 2);
	if(RctPtInRectangle(tap.x, tap.y, &area)) {
		FldAddPoints(SIF_ACount);
		return;
	}

	RctSetRectangle(&area, 28, 24, 2, 3);
	if(RctPtInRectangle(tap.x, tap.y, &area)) {
		FldAddPoints(SIF_ACount);
		return;
	}

	RctSetRectangle(&area, 37, 24, 2, 3);
	if(RctPtInRectangle(tap.x, tap.y, &area)) {
		FldAddPoints(SIF_ACount);
		return;
	}

	RctSetRectangle(&area, 29, 21, 1, 3);
	if(RctPtInRectangle(tap.x, tap.y, &area)) {
		FldAddPoints(SIF_ACount);
		return;
	}

	RctSetRectangle(&area, 37, 21, 1, 3);
	if(RctPtInRectangle(tap.x, tap.y, &area)) {
		FldAddPoints(SIF_ACount);
		return;
	}

	RctSetRectangle(&area, 23, 39, 2, 3);
	if(RctPtInRectangle(tap.x, tap.y, &area)) {
		FldAddPoints(SIF_ACount);
		return;
	}

	RctSetRectangle(&area, 42, 39, 2, 3);
	if(RctPtInRectangle(tap.x, tap.y, &area)) {
		FldAddPoints(SIF_ACount);
		return;
	}

	RctSetRectangle(&area, 24, 36, 1, 3);
	if(RctPtInRectangle(tap.x, tap.y, &area)) {
		FldAddPoints(SIF_ACount);
		return;
	}

	RctSetRectangle(&area, 42, 36, 1, 3);
	if(RctPtInRectangle(tap.x, tap.y, &area)) {
		FldAddPoints(SIF_ACount);
		return;
	}

	RctSetRectangle(&area, 24, 58, 1, 3);
	if(RctPtInRectangle(tap.x, tap.y, &area)) {
		FldAddPoints(SIF_ACount);
		return;
	}

	RctSetRectangle(&area, 42, 58, 1, 3);
	if(RctPtInRectangle(tap.x, tap.y, &area)) {
		FldAddPoints(SIF_ACount);
		return;
	}

	if(tap.x == 23 && tap.y == 58) {
		FldAddPoints(SIF_ACount);
		return;
	} else if (tap.x == 43 && tap.y == 58) {
		FldAddPoints(SIF_ACount);
		return;
	} else if (tap.x == 29 && tap.y == 66) {
		FldAddPoints(SIF_ACount);
		return;
	} else if (tap.x == 37 && tap.y == 66) {
		FldAddPoints(SIF_ACount);
		return;
	}

	/*
	** Check tap in the C area
	*/

	RctSetRectangle(&area, 10, 44, 47, 22);
	if(RctPtInRectangle(tap.x, tap.y, &area)) {
		FldAddPoints(SIF_CCount);
		return;
	}

	RctSetRectangle(&area, 21, 17, 25, 62);
	if(RctPtInRectangle(tap.x, tap.y, &area)) {
		FldAddPoints(SIF_CCount);
		return;
	}

	RctSetRectangle(&area, 17, 26, 33, 48);
	if(RctPtInRectangle(tap.x, tap.y, &area)) {
		FldAddPoints(SIF_CCount);
		return;
	}

	RctSetRectangle(&area, 11, 41, 6, 3);
	if(RctPtInRectangle(tap.x, tap.y, &area)) {
		FldAddPoints(SIF_CCount);
		return;
	}

	RctSetRectangle(&area, 50, 41, 6, 3);
	if(RctPtInRectangle(tap.x, tap.y, &area)) {
		FldAddPoints(SIF_CCount);
		return;
	}

	RctSetRectangle(&area, 11, 66, 6, 1);
	if(RctPtInRectangle(tap.x, tap.y, &area)) {
		FldAddPoints(SIF_CCount);
		return;
	}

	RctSetRectangle(&area, 50, 66, 6, 1);
	if(RctPtInRectangle(tap.x, tap.y, &area)) {
		FldAddPoints(SIF_CCount);
		return;
	}

	RctSetRectangle(&area, 12, 38, 5, 3);
	if(RctPtInRectangle(tap.x, tap.y, &area)) {
		FldAddPoints(SIF_CCount);
		return;
	}
	RctSetRectangle(&area, 50, 41, 6, 3);
	if(RctPtInRectangle(tap.x, tap.y, &area)) {
		FldAddPoints(SIF_CCount);
		return;
	}

	RctSetRectangle(&area, 11, 66, 6, 1);
	if(RctPtInRectangle(tap.x, tap.y, &area)) {
		FldAddPoints(SIF_CCount);
		return;
	}

	RctSetRectangle(&area, 50, 66, 6, 1);
	if(RctPtInRectangle(tap.x, tap.y, &area)) {
		FldAddPoints(SIF_CCount);
		return;
	}

	RctSetRectangle(&area, 12, 38, 5, 3);
	if(RctPtInRectangle(tap.x, tap.y, &area)) {
		FldAddPoints(SIF_CCount);
		return;
	}

	RctSetRectangle(&area, 50, 38, 5, 3);
	if(RctPtInRectangle(tap.x, tap.y, &area)) {
		FldAddPoints(SIF_CCount);
		return;
	}

	RctSetRectangle(&area, 12, 67, 5, 1);
	if(RctPtInRectangle(tap.x, tap.y, &area)) {
		FldAddPoints(SIF_CCount);
		return;
	}

	RctSetRectangle(&area, 50, 67, 5, 1);
	if(RctPtInRectangle(tap.x, tap.y, &area)) {
		FldAddPoints(SIF_CCount);
		return;
	}

	RctSetRectangle(&area, 13, 36, 4, 2);
	if(RctPtInRectangle(tap.x, tap.y, &area)) {
		FldAddPoints(SIF_CCount);
		return;
	}

	RctSetRectangle(&area, 50, 36, 4, 2);
	if(RctPtInRectangle(tap.x, tap.y, &area)) {
		FldAddPoints(SIF_CCount);
		return;
	}

	RctSetRectangle(&area, 13, 68, 4, 2);
	if(RctPtInRectangle(tap.x, tap.y, &area)) {
		FldAddPoints(SIF_CCount);
		return;
	}

	RctSetRectangle(&area, 50, 68, 4, 2);
	if(RctPtInRectangle(tap.x, tap.y, &area)) {
		FldAddPoints(SIF_CCount);
		return;
	}

	RctSetRectangle(&area, 14, 34, 3, 2);
	if(RctPtInRectangle(tap.x, tap.y, &area)) {
		FldAddPoints(SIF_CCount);
		return;
	}

	RctSetRectangle(&area, 50, 34, 3, 2);
	if(RctPtInRectangle(tap.x, tap.y, &area)) {
		FldAddPoints(SIF_CCount);
		return;
	}

	RctSetRectangle(&area, 15, 70, 2, 2);
	if(RctPtInRectangle(tap.x, tap.y, &area)) {
		FldAddPoints(SIF_CCount);
		return;
	}

	RctSetRectangle(&area, 50, 70, 2, 2);
	if(RctPtInRectangle(tap.x, tap.y, &area)) {
		FldAddPoints(SIF_CCount);
		return;
	}

	RctSetRectangle(&area, 15, 31, 2, 3);
	if(RctPtInRectangle(tap.x, tap.y, &area)) {
		FldAddPoints(SIF_CCount);
		return;
	}

	RctSetRectangle(&area, 50, 31, 2, 3);
	if(RctPtInRectangle(tap.x, tap.y, &area)) {
		FldAddPoints(SIF_CCount);
		return;
	}

	RctSetRectangle(&area, 18, 74, 3, 2);
	if(RctPtInRectangle(tap.x, tap.y, &area)) {
		FldAddPoints(SIF_CCount);
		return;
	}

	RctSetRectangle(&area, 46, 74, 3, 2);
	if(RctPtInRectangle(tap.x, tap.y, &area)) {
		FldAddPoints(SIF_CCount);
		return;
	}

	RctSetRectangle(&area, 23, 79, 21, 3);
	if(RctPtInRectangle(tap.x, tap.y, &area)) {
		FldAddPoints(SIF_CCount);
		return;
	}

	RctSetRectangle(&area, 22, 15, 23, 2);
	if(RctPtInRectangle(tap.x, tap.y, &area)) {
		FldAddPoints(SIF_CCount);
		return;
	}

	RctSetRectangle(&area, 24, 82, 19, 1);
	if(RctPtInRectangle(tap.x, tap.y, &area)) {
		FldAddPoints(SIF_CCount);
		return;
	}

	RctSetRectangle(&area, 25, 83, 17, 1);
	if(RctPtInRectangle(tap.x, tap.y, &area)) {
		FldAddPoints(SIF_CCount);
		return;
	}

	RctSetRectangle(&area, 19, 22, 2, 4);
	if(RctPtInRectangle(tap.x, tap.y, &area)) {
		FldAddPoints(SIF_CCount);
		return;
	}

	RctSetRectangle(&area, 46, 22, 2, 4);
	if(RctPtInRectangle(tap.x, tap.y, &area)) {
		FldAddPoints(SIF_CCount);
		return;
	}

	RctSetRectangle(&area, 19, 76, 2, 1);
	if(RctPtInRectangle(tap.x, tap.y, &area)) {
		FldAddPoints(SIF_CCount);
		return;
	}

	RctSetRectangle(&area, 46, 76, 2, 1);
	if(RctPtInRectangle(tap.x, tap.y, &area)) {
		FldAddPoints(SIF_CCount);
		return;
	}
	RctSetRectangle(&area, 20, 19, 1, 3);
	if(RctPtInRectangle(tap.x, tap.y, &area)) {
		FldAddPoints(SIF_CCount);
		return;
	}

	RctSetRectangle(&area, 46, 19, 1, 3);
	if(RctPtInRectangle(tap.x, tap.y, &area)) {
		FldAddPoints(SIF_CCount);
		return;
	}

	RctSetRectangle(&area, 18, 24, 1, 2);
	if(RctPtInRectangle(tap.x, tap.y, &area)) {
		FldAddPoints(SIF_CCount);
		return;
	}

	RctSetRectangle(&area, 48, 24, 1, 2);
	if(RctPtInRectangle(tap.x, tap.y, &area)) {
		FldAddPoints(SIF_CCount);
		return;
	}

	RctSetRectangle(&area, 16, 29, 1, 2);
	if(RctPtInRectangle(tap.x, tap.y, &area)) {
		FldAddPoints(SIF_CCount);
		return;
	}

	RctSetRectangle(&area, 50, 29, 1, 2);
	if(RctPtInRectangle(tap.x, tap.y, &area)) {
		FldAddPoints(SIF_CCount);
		return;
	}

	if(tap.x == 14 && tap.y == 70) {
		FldAddPoints(SIF_CCount);
		return;
	} else if (tap.x == 16 && tap.y == 72) {
		FldAddPoints(SIF_CCount);
		return;
	} else if (tap.x == 52 && tap.y == 70) {
		FldAddPoints(SIF_CCount);
		return;
	} else if (tap.x == 50 && tap.y == 72) {
		FldAddPoints(SIF_CCount);
		return;
	} else if (tap.x == 20 && tap.y == 77) {
		FldAddPoints(SIF_CCount);
		return;
	} else if (tap.x == 22 && tap.y == 79) {
		FldAddPoints(SIF_CCount);
		return;
	} else if (tap.x == 46 && tap.y == 77) {
		FldAddPoints(SIF_CCount);
		return;
	} else if (tap.x == 44 && tap.y == 79) {
		FldAddPoints(SIF_CCount);
		return;
	}

	/*
	** Check tap in the C area
	*/

	RctSetRectangle(&area, 0, 44, 67, 29);
	if(RctPtInRectangle(tap.x, tap.y, &area)) {
		FldAddPoints(SIF_DCount);
		return;
	}

	RctSetRectangle(&area, 22, 84, 23, 18);
	if(RctPtInRectangle(tap.x, tap.y, &area)) {
		FldAddPoints(SIF_DCount);
		return;
	}

	RctSetRectangle(&area, 8, 58, 51, 11);
	if(RctPtInRectangle(tap.x, tap.y, &area)) {
		FldAddPoints(SIF_DCount);
		return;
	}

	RctSetRectangle(&area, 5, 37, 57, 4);
	if(RctPtInRectangle(tap.x, tap.y, &area)) {
		FldAddPoints(SIF_DCount);
		return;
	}

	RctSetRectangle(&area, 5, 76, 57, 4);
	if(RctPtInRectangle(tap.x, tap.y, &area)) {
		FldAddPoints(SIF_DCount);
		return;
	}

	RctSetRectangle(&area, 14, 88, 39, 4);
	if(RctPtInRectangle(tap.x, tap.y, &area)) {
		FldAddPoints(SIF_DCount);
		return;
	}

	RctSetRectangle(&area, 2, 41, 6, 35);
	if(RctPtInRectangle(tap.x, tap.y, &area)) {
		FldAddPoints(SIF_DCount);
		return;
	}

	RctSetRectangle(&area, 59, 41, 6, 35);
	if(RctPtInRectangle(tap.x, tap.y, &area)) {
		FldAddPoints(SIF_DCount);
		return;
	}

	RctSetRectangle(&area, 11, 29, 45, 4);
	if(RctPtInRectangle(tap.x, tap.y, &area)) {
		FldAddPoints(SIF_DCount);
		return;
	}

	RctSetRectangle(&area, 11, 84, 45, 4);
	if(RctPtInRectangle(tap.x, tap.y, &area)) {
		FldAddPoints(SIF_DCount);
		return;
	}

	RctSetRectangle(&area, 17, 92, 33, 4);
	if(RctPtInRectangle(tap.x, tap.y, &area)) {
		FldAddPoints(SIF_DCount);
		return;
	}

	RctSetRectangle(&area, 14, 25, 39, 4);
	if(RctPtInRectangle(tap.x, tap.y, &area)) {
		FldAddPoints(SIF_DCount);
		return;
	}

	RctSetRectangle(&area, 17, 21, 33, 4);
	if(RctPtInRectangle(tap.x, tap.y, &area)) {
		FldAddPoints(SIF_DCount);
		return;
	}

	RctSetRectangle(&area, 20, 96, 27, 4);
	if(RctPtInRectangle(tap.x, tap.y, &area)) {
		FldAddPoints(SIF_DCount);
		return;
	}

	RctSetRectangle(&area, 20, 17, 1, 2);
	if(RctPtInRectangle(tap.x, tap.y, &area)) {
		FldAddPoints(SIF_DCount);
		return;
	}

	RctSetRectangle(&area, 46, 17, 1, 2);
	if(RctPtInRectangle(tap.x, tap.y, &area)) {
		FldAddPoints(SIF_DCount);
		return;
	}

	for(x = 19, y = 19; x > 3; x -= 3, y += 4) {
		RctSetRectangle(&area, x, y, 1, 2);
		if(RctPtInRectangle(tap.x, tap.y, &area)) {
			FldAddPoints(SIF_DCount);
			return;
		}
	}

	for(x = 21, y = 16; x > 0; x -= 3, y += 4)
		if(tap.x == x && tap.y == y) {
			FldAddPoints(SIF_DCount);
			return;
		}

	for(x = 47, y = 19; x < 64; x += 3, y += 4) {
		RctSetRectangle(&area, x, y, 1, 2);
		if(RctPtInRectangle(tap.x, tap.y, &area)) {
			FldAddPoints(SIF_DCount);
			return;
		}
	}

	for(x = 45, y = 16; x < 66; x += 3, y += 4)
		if(tap.x == x && tap.y == y) {
			FldAddPoints(SIF_DCount);
			return;
		}

	for(x = 62, y = 76; x > 45; x -= 3, y -= 4) {
		RctSetRectangle(&area, x, y, 1, 2);
		if(RctPtInRectangle(tap.x, tap.y, &area)) {
			FldAddPoints(SIF_DCount);
			return;
		}
	}

	for(x = 65, y = 73; x > 44; x -= 3, y -= 4)
		if(tap.x == x && tap.y == y) {
			FldAddPoints(SIF_DCount);
			return;
		}

	for(x = 4, y = 76; x < 20; x += 3, y += 4) {
		RctSetRectangle(&area, x, y, 1, 2);
		if(RctPtInRectangle(tap.x, tap.y, &area)) {
			FldAddPoints(SIF_DCount);
			return;
		}
	}

	for(x = 2, y = 73; x < 22; x += 3, y += 4)
		if(tap.x == x && tap.y == y) {
			FldAddPoints(SIF_DCount);
			return;
		}

	/* Misses */

	RctSetRectangle(&area, 0, 15, 67, 29);
	if(RctPtInRectangle(tap.x, tap.y, &area)) {
		FldAddPoints(SIF_MCount);
		return;
	}

	/* No Shoots */

	RctSetRectangle(&area, 0, 73, 22, 29);
	if(RctPtInRectangle(tap.x, tap.y, &area)) {
		FldAddPoints(SIF_NCount);
		return;
	}

	/* Penalties */

	RctSetRectangle(&area, 45, 73, 22, 29);
	if(RctPtInRectangle(tap.x, tap.y, &area)) {
		FldAddPoints(SIF_PCount);
		return;
	}
}

static void ScoreInputFormSaveRecord()
{
	ScorePtr record;

	record = MemHandleLock(editrecordH);
	ErrFatalDisplayIf(record == NULL, NPtr);

	/*
	 * Get the values back to the unpacked structures and pass
	 * to the specialized function to pack in the datagbase.
	 */

	record->As = NumFldGetInt(SIF_ACount);
	record->Cs = NumFldGetInt(SIF_CCount);
	record->Ds = NumFldGetInt(SIF_DCount);
	record->Misses = NumFldGetInt(SIF_MCount);
	record->PTs = NumFldGetInt(SIF_NCount);
	record->Procs = NumFldGetInt(SIF_PCount);
	record->Time = NumFldGetDouble(SIF_TCount);

	if((record->As + record->Cs + record->Ds + record->Misses) != StageDBStageRounds(matchrecordindex, stagelistindex))
	{
		UInt16 option;

		option = FrmAlert(WrongScore_Alert);
		if(option == 1)	return;
	}

	ScoreDBUpdateScore(record, matchrecordindex, complistindex, stagelistindex);
	MemHandleUnlock(editrecordH);
	FrmGotoForm(CompListForm);
}

Boolean ScoreInputFormHandleEvent(EventPtr e)
{
	Boolean handled = false;
	FormPtr frm;
	FieldPtr fld;
	UInt16 objIndex, objID, i;
	RectangleType r;

	switch (e->eType) {

	case frmOpenEvent:
		frm = FrmGetActiveForm();
		ScoreInputFormSetup(frm);
		handled = true;
		break;

	case frmCloseEvent:
		for(i = SIF_ACount; i <= SIF_TCount; i++)
			FldDestroy(i);
		NEFldDestroy(SIF_Points);
		NEFldDestroy(SIF_HFResult);
		MemHandleFree(editrecordH);
		break;

	case menuEvent:
		MenuEraseStatus(NULL);

		switch(e->data.menu.itemID) {
		case About_Menu:
			FrmCustomAlert(About_Alert, NULL, NULL, NULL);
			handled = true;
			break;

		case SIF_Reset_Menu:
			ScoreInputFormReset();
			handled = true;
			break;

		default:
			handled = false;
			break;
		}

		break;

	case ctlSelectEvent:
		switch(e->data.ctlSelect.controlID) {
		case SIF_Ok:
			ScoreInputFormSaveRecord();
			handled = true;
			break;

		case SIF_Cancel:
			FrmGotoForm(CompListForm);
			handled = true;
			break;

		case SIF_Reset:
			ScoreInputFormReset();
			handled = true;
			break;
		}
	break;

	case keyDownEvent:
		frm = FrmGetActiveForm();
		fld = FrmGetObjectPtr(frm, FrmGetFocus(frm));
		objID = FrmGetObjectId(frm, FrmGetFocus(frm));
		if(objID >= SIF_ACount && objID <= SIF_TCount) {
			handled = FldHandleEvent(fld, e);
			ScoreInputFormRecompute();
		}
		break;

	case penDownEvent:
		frm = FrmGetActiveForm();
		objIndex = FrmGetObjectIndex(frm, IPSCTarget);
		FrmGetObjectBounds(frm, objIndex, &r);
		if(RctPtInRectangle(e->screenX, e->screenY, &r)) {
			PointType tap;
			tap.x = e->screenX;
			tap.y = e->screenY;
			TargetHit(tap);
			ScoreInputFormRecompute();
			handled = true;
		}
		break;

	default:
	break;
	}

	return handled;
}
