/*
 *
 * Copyright (c) 2004-2005 Guido Falsi <mad@madpilot.net>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: StageList.c,v 1.21 2005/03/13 12:22:09 mad Exp $
 *
 */

#include <PalmOS.h>

#include "miscdefs.h"
#include "PalmSS.h"
#include "PSSdb.h"
#include "MatchList.h"
#include "PalmSSRsc.h"
#include "Common.h"

#define StageNewFormGetMinRounds() (NumFldGetInt(SNF_Irons) + NumFldGetInt(SNF_Papers) * 2 + NumFldGetInt(SNF_Disapp) * 2)

#define StageNewFormGetMaxPoints() (StageNewFormGetMinRounds() * 5)

static char			*NPtr = NULLPTRERR;
static char			*NHdl = NULLHDLERR;

Int16				stagelistindex = noListSelection;

static MemHandle		editrecordH;
static Boolean			editisnew = false;

/* pointer for the table information array */

static StageTableDataPtr	TableArray = NULL;

/* global variables to track the table state */

static Int16			selectedItem = NoRecord;
static Int16			topItem = 0;
static Int16			selectedRow = NoRecord;
static Int16			selectedCol = NoRecord;
static Boolean			selectedIsOffScreen = false;

static void StageTblDraw(void *table, Int16 row, Int16 col, RectanglePtr bounds)
{
	Char mpstr[maxStrIToALen];
	UInt16 len, width, ArrayItem, numrecs;
	Coord x;

	ArrayItem = row + topItem;
	numrecs = StageInMatchDBNumRecords(matchrecordindex);

	if(ArrayItem < numrecs)
	{
		if(col == Name)
		{
			WinDrawChars(TableArray[ArrayItem].name, StrLen(TableArray[ArrayItem].name), bounds->topLeft.x, bounds->topLeft.y);
		} else if(col == MaxPts) {
			StrIToA(mpstr, TableArray[ArrayItem].mpts);
			len = StrLen(mpstr);
			width = FntCharsWidth(mpstr, len);
			x = bounds->topLeft.x + bounds->extent.x - width - 2;
			WinDrawChars(mpstr, len, x, bounds->topLeft.y);
		}
	}
}

static void StageListFormSetup(FormPtr frm)
{
	TablePtr tbl;
	ScrollBarPtr scroller;
	UInt16 numrecs, i, rows;

	selectedItem = NoRecord;
	topItem = 0;
	selectedRow = NoRecord;
	selectedCol = NoRecord;
	selectedIsOffScreen = false;
	TableArray = NULL;

	tbl = GetObjectPtr(StagesList);
	scroller = GetObjectPtr(StagesList_Scroll);
	rows = TblGetNumberOfRows(tbl);

	for(i = 0; i < rows; i++)
	{
		TblSetItemStyle(tbl, i, Name, customTableItem);
		TblSetItemStyle(tbl, i, MaxPts, customTableItem);

		TblSetRowUsable(tbl, i, true);
	}

	TblSetColumnUsable(tbl, Name, true);
	TblSetColumnUsable(tbl, MaxPts, true);

	TblSetColumnSpacing(tbl, Name, 2);

	TblSetCustomDrawProcedure(tbl, Name, StageTblDraw);
	TblSetCustomDrawProcedure(tbl, MaxPts, StageTblDraw);

	numrecs = StageInMatchDBNumRecords(matchrecordindex);

	StageDBTableDataQuery(matchrecordindex, &TableArray); /* XXX */

	SclSetScrollBar(scroller, topItem, 0, (numrecs < rows) ? 0 : (numrecs - rows), rows);
}

static void StageListFormNewStage()
{
	StagePtr newstage;

	newstage = MemPtrNew(sizeof(Stage));
	ErrFatalDisplayIf(newstage == NULL, NPtr);

	/* Initialize the sctructure with default(mostly null) values */

	newstage->num = 0;
	newstage->papers = 0;
	newstage->irons = 0;
	newstage->disapp = 0;
	newstage->penalties = 0;
	newstage->MtcRef = GetMatchUID(matchrecordindex);
	newstage->prev = StageDBGetLastUID(matchrecordindex);
	newstage->next = NoRecord;
	StrCopy(newstage->name, "");

	StageDBCreateStage(newstage, matchrecordindex, &stagelistindex);
	MemPtrFree(newstage);

	editisnew = true;

	FrmGotoForm(StageNewForm);
}

static void StageListFormDelStage()
{
	TablePtr tbl;
	ScrollBarPtr scroller;
	UInt16 numrecs, page = 0;

	stagelistindex = selectedItem;
	if(stagelistindex == NoRecord) return;

	StageDBTableDataFree(matchrecordindex, &TableArray, false);

	StageDBDeleteStage(matchrecordindex, stagelistindex);

	/* Now we should update the list and scroller. */
	tbl = GetObjectPtr(StagesList);
	scroller = GetObjectPtr(StagesList_Scroll);
	numrecs = StageInMatchDBNumRecords(matchrecordindex);
	page = TblGetNumberOfRows(tbl);

	if(numrecs > page)
		topItem = ((numrecs - topItem - 1) < page) ? (numrecs - page) :
topItem;

	selectedItem = NoRecord;
	selectedRow = NoRecord;
	selectedCol = NoRecord;
	selectedIsOffScreen = false;

	SclSetScrollBar(scroller, topItem, 0, (numrecs < page) ? 0 : (numrecs - page), page);
	StageDBTableDataQuery(matchrecordindex, &TableArray);
	TblMarkTableInvalid(tbl);
	TblRedrawTable(tbl);
}

Boolean StageListFormHandleEvent(EventPtr e)
{
	Boolean handled = false;
	FormPtr frm;

	switch (e->eType) {

	case frmOpenEvent:
		frm = FrmGetActiveForm();
		StageListFormSetup(frm);
		FrmDrawForm(frm);
		handled = true;
		break;

	case frmCloseEvent:
		StageDBTableDataFree(matchrecordindex, &TableArray, editisnew);
		break;

	case menuEvent:
		MenuEraseStatus(NULL);

		switch(e->data.menu.itemID) {

		case About_Menu:
			FrmCustomAlert(About_Alert, NULL, NULL, NULL);
			handled = true;
			break;

		default:
			handled = false;
			break;
		}

		break;

	case ctlSelectEvent:
		switch(e->data.ctlSelect.controlID) {
		case SLF_Prev:
			stagelistindex = NoRecord;
			FrmGotoForm(MatchListForm);
			handled = true;
			break;

		case SLF_New:
			StageListFormNewStage();
			handled = true;
			break;

		case SLF_Edit:
			stagelistindex = selectedItem;
			if(stagelistindex == NoRecord) break;
			editisnew = false;
			FrmGotoForm(StageNewForm);
			handled = true;
			break;

		case SLF_Del:
			StageListFormDelStage();
			handled = true;
			break;

		case SLF_Next:
			stagelistindex = selectedItem;
			if(stagelistindex == NoRecord) break;
			FrmGotoForm(CompListForm);
			handled = true;
			break;

		case SLF_Results:
			FrmGotoForm(ResultsForm);
			handled = true;
			break;
		}
		break;

	case sclRepeatEvent:
		ListFormScrollTable(GetObjectPtr(StagesList), GetObjectPtr(StagesList_Scroll), &topItem, &selectedItem, &selectedRow, &selectedIsOffScreen, e, Scroller);
		break;

	case tblEnterEvent:
		TblSelectTableItem(GetObjectPtr(StagesList), StageInMatchDBNumRecords(matchrecordindex), &topItem, &selectedItem, &selectedRow, &selectedCol, &selectedIsOffScreen, e);
		handled=true;
		break;

	case keyDownEvent:
		switch(e->data.keyDown.chr) {
		case pageDownChr:
			ListFormScrollTable(GetObjectPtr(StagesList), GetObjectPtr(StagesList_Scroll), &topItem, &selectedItem, &selectedRow, &selectedIsOffScreen, e, BtnDn);
			handled = true;
			break;

		case pageUpChr:
			ListFormScrollTable(GetObjectPtr(StagesList), GetObjectPtr(StagesList_Scroll), &topItem, &selectedItem, &selectedRow, &selectedIsOffScreen, e, BtnUp);
			handled = true;
			break;
		}
		break;

	default:
		break;

	}

	return handled;
}

static void StageNewFormSetup(FormPtr frm)
{
	StagePtr record;
	FieldPtr fld;
	UInt32 offset;

	editrecordH = MemHandleNew(sizeof(Stage));
	ErrFatalDisplayIf(editrecordH == NULL, NHdl);
	record = MemHandleLock(editrecordH);
	ErrFatalDisplayIf(record == NULL, NPtr);

	StageDBGetStage(matchrecordindex, stagelistindex, record);

	fld = GetObjectPtr(SNF_Name);
	offset = sizeof(UInt16) + sizeof(UInt8) * 4 + sizeof(UInt32) * 3;
	FldSetText(fld, editrecordH, offset, BUFSIZE);

	NumFldSetInt(SNF_Number, record->num);
	NumFldSetInt(SNF_Papers, record->papers);
	NumFldSetInt(SNF_Irons, record->irons);
	NumFldSetInt(SNF_Disapp, record->disapp);
	NumFldSetInt(SNF_Penalties, record->penalties);

	MemHandleUnlock(editrecordH);

	NEFldNumSetInt(SNF_MinRounds, StageNewFormGetMinRounds());
	NEFldNumSetInt(SNF_MaxPoints, StageNewFormGetMaxPoints());

	FrmSetFocus(frm, FrmGetObjectIndex(frm, SNF_Name));
}

static void StageNewFormRecompute()
{
	FieldPtr fld;

	fld = NEFldNumSetInt(SNF_MinRounds, StageNewFormGetMinRounds());
	FldDrawField(fld);

	fld = NEFldNumSetInt(SNF_MaxPoints, StageNewFormGetMaxPoints());
	FldDrawField(fld);
}

static void StageNewFormSaveRecord()
{
	StagePtr record;

	record = MemHandleLock(editrecordH);
	ErrFatalDisplayIf(record == NULL, NPtr);

	/*
	 * Get the values back to the unpacked structures and pass
	 * to the specialized function to pack in the datagbase.
	 */

	record->num = NumFldGetInt(SNF_Number);
	record->papers = NumFldGetInt(SNF_Papers);
	record->irons = NumFldGetInt(SNF_Irons);
	record->disapp = NumFldGetInt(SNF_Disapp);
	record->penalties = NumFldGetInt(SNF_Penalties);

	StageDBUpdateStage(record, matchrecordindex, stagelistindex);
	MemHandleUnlock(editrecordH);
}

Boolean StageNewFormHandleEvent(EventPtr e)
{
	Boolean handled = false;
	FormPtr frm;
	FieldPtr fld;
	UInt16 objID;

	switch (e->eType) {

	case frmOpenEvent:
		frm = FrmGetActiveForm();
		StageNewFormSetup(frm);
		FrmDrawForm(frm);
		handled = true;
		break;

	case frmCloseEvent:
		if(editisnew) StageDBDeleteStage(matchrecordindex, stagelistindex);
		FldDestroy(SNF_Name);
		FldDestroy(SNF_Number);
		FldDestroy(SNF_Papers);
		FldDestroy(SNF_Irons);
		FldDestroy(SNF_Disapp);
		FldDestroy(SNF_Penalties);
		NEFldDestroy(SNF_MinRounds);
		NEFldDestroy(SNF_MaxPoints);
		editisnew = false;
		break;

	case menuEvent:
		MenuEraseStatus(NULL);

		switch(e->data.menu.itemID) {

		case About_Menu:
			FrmCustomAlert(About_Alert, NULL, NULL, NULL);
			handled = true;
			break;

		default:
			handled = false;
			break;
		}

		break;

	case ctlSelectEvent:
		switch(e->data.ctlSelect.controlID) {

		case SNF_Cancel:
			FrmGotoForm(StageListForm);
			handled = true;
			break;

		case SNF_Ok:
			StageNewFormSaveRecord();
			editisnew = false;
			FrmGotoForm(StageListForm);
			handled = true;
			break;

		}
		break;

	case keyDownEvent:
		frm = FrmGetActiveForm();
		fld = FrmGetObjectPtr(frm, FrmGetFocus(frm));
		objID = FrmGetObjectId(frm, FrmGetFocus(frm));
		if(objID == SNF_Papers ||
			objID == SNF_Irons ||
			objID == SNF_Disapp)
		{
			handled = FldHandleEvent(fld, e);
			StageNewFormRecompute();
		}
		break;

	default:
		break;

	}

	return handled;
}
