/*
 *
 * Copyright (c) 2004-2005 Guido Falsi <mad@madpilot.net>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: miscdefs.h,v 1.14 2005/03/12 05:02:42 mad Exp $
 *
 */

#ifndef _MISCDEFS_H_
#define _MISCDEFS_H_

#define BUFSIZE			31

#define NULLPTRERR		"NULL Pointer returned."
#define NULLHDLERR		"NULL Handler returned."

#define appID			'TDSF'
#define appPrefID		0
#define appPrefVer		1

#define NoRecord		-1

/* Columns Mnemonics */
#define Place			0
#define Date			1
#define Name			0
#define MaxPts			1
#define CompUnused		1
#define Position		0
#define RName			1
#define Percent			2

/* how to scroll constants */
#define Scroller		0
#define BtnUp			1
#define BtnDn			2

/* for the time being simply set this to zero. How do you really use this???? */
#define DBcardNo		0
#define MatchDBType		'mhDB'
#define MatchDBName		"PalmSS Matches DB"
#define StageDBType		'seDB'
#define StageDBName		"PalmSS Stages DB"
#define CompDBType		'cpDB'
#define CompDBName		"PalmSS Competitors DB"
#define ScoreDBType		'srDB'
#define ScoreDBName		"PalmSS Scores DB"

/* Competitor's properties constants */

/* Divisions */
#define Production		0
#define Standard		1
#define Modified		2
#define Open			3

/* Power Factor */
#define Minor			0
#define Major			1

/* typedefs */

/*
 * structures used in array form to return information to be put
 * in the form's tables
 */

typedef struct {
	char *place;
	DatePtr date;
	MemHandle recordH;
} MatchTableData;

typedef MatchTableData * MatchTableDataPtr;

typedef struct {
	char *name;
	UInt16 mpts;
	MemHandle recordH;
} StageTableData;

typedef StageTableData * StageTableDataPtr;

typedef struct {
	char *name;
	Boolean DQ;
	MemHandle recordH;
} CompTableData;

typedef CompTableData * CompTableDataPtr;

/*
 * Here I'm playing a bit dirty on the fact that the packed and
 * unpacked Score structures are the same.
 */

typedef struct {
	double HF;
	Int16 comp;
	MemHandle ScoreDBrH; 
} ScoreTableData;

typedef ScoreTableData * ScoreTableDataPtr;

typedef struct {
	UInt16 position;
	Char *name;
	double percent;
	double matchpoints;
	MemHandle CompDBrH;
} ResultTableData;

typedef ResultTableData * ResultTableDataPtr;

/* unpacked records structures */

/*
 * NOTE packed records are manged by PSSdb.c and their structures
 * are not to be known to the rest of the application
 */

typedef struct {
	DateType date;
	UInt32 StgRef;
	UInt32 CptRef;
	UInt16 StgCnt;
	UInt16 CptCnt;
	Char place[BUFSIZE];
	Char name[BUFSIZE];
} Match;

typedef Match * MatchPtr;

typedef struct {
	UInt16 num;
	UInt8 papers;
	UInt8 irons;
	UInt8 disapp;
	UInt8 penalties;
	UInt32 MtcRef;
	UInt32 prev;
	UInt32 next;
	Char name[BUFSIZE];
} Stage;

typedef Stage * StagePtr;

typedef struct {
	UInt16 num;
	UInt16 division:2;
	UInt16 PF:1;
	UInt16 DQ:1;
	UInt32 MtcRef;
	UInt32 ScrRef;
	UInt32 prev;
	UInt32 next;
	Char name[BUFSIZE];
} Comp;

typedef Comp * CompPtr;

typedef struct {
	UInt8 As;
	UInt8 Cs;
	UInt8 Ds;
	UInt8 PTs;
	UInt8 Misses;
	UInt8 Procs;
	double Time;
	UInt16 Zeroed:1; /* Unused */
	UInt16 Scored:1; /* set when necessary but never read */
	UInt32 StgRef;
	UInt32 CptRef;
	UInt32 prev;
	UInt32 next;
} Score;

typedef Score * ScorePtr;

#endif /* ndef _MISCDEFS_H_ */
